# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Defined types

* [`extended_stdlib::script`](#extended_stdlibscript): Manages a script's existence in the directory of a system, and then executes the script if the script's attributes changed.

### Functions

* [`extended_stdlib::ascii_only`](#extended_stdlibascii_only): Returns true for a string which has only ASCII characters, and false otherwise.
* [`extended_stdlib::center`](#extended_stdlibcenter): Centers a string in width. If the width is greater than the length of the string, then the string is centered by padding and padded with pads
* [`extended_stdlib::compact`](#extended_stdlibcompact): Returns a hash or array with all of the undef value entries removed
* [`extended_stdlib::decrypt_string`](#extended_stdlibdecrypt_string): Returns a decrypted String using the AES-256-CBC OpenSSL cipher algorithm.
* [`extended_stdlib::drop`](#extended_stdlibdrop): Returns an Array containing all but the first num element of the array, where num is a non-negative Integer
* [`extended_stdlib::end_with`](#extended_stdlibend_with): Returns true if the string ends with one of the suffixes given.
* [`extended_stdlib::imperative`](#extended_stdlibimperative): THIS FUNCTION IS CURRENTLY IN BETA. Puppet function to simulate imperative execution for a subset of resources by constructing iterative dependencies of each resource upon the previous resource. This thusly ensures consecutive resource application vis a vis imperative application instead of declarative.
* [`extended_stdlib::intersect`](#extended_stdlibintersect): Returns true if the array and other_array have at least one element in common, otherwise returns false. Requires Ruby >= 3.0.
* [`extended_stdlib::minmax`](#extended_stdlibminmax): Returns a new two element Array containing the minimum and maximum values from an array of integers
* [`extended_stdlib::none`](#extended_stdlibnone): Returns a boolean of whether no elements of an array, or array or hash with a lambda block meet a given criterion. Note that the combined blo
* [`extended_stdlib::product`](#extended_stdlibproduct): Computes and returns all combinations of elements from all of the Arrays
* [`extended_stdlib::sample`](#extended_stdlibsample): Returns random elements from an Array. Note that the return is non-deterministic.
* [`extended_stdlib::start_with`](#extended_stdlibstart_with): Returns true if the string starts with one of the prefixes given. Each prefix should be a String or (Ruby >= 2.5) Regexp.
* [`extended_stdlib::subset`](#extended_stdlibsubset): Returns a boolean of whether the first hash is a subset of the second hash
* [`extended_stdlib::sum`](#extended_stdlibsum): Returns the sum of an optional summand with all elements of an Array.
* [`extended_stdlib::transpose`](#extended_stdlibtranspose): Transposes the rows and columns in an Array of Arrays; the nested Arrays must all be the same size.

### Tasks

* [`csr_attributes`](#csr_attributes): Modifies CSR attributes on a client server.

### Plans

* [`extended_stdlib::csr_regenerate`](#extended_stdlibcsr_regenerate): This plan modifies the CSR attributes on servers, and then regenerates the client certificate on the servers to enable PuppetDB to update the

## Defined types

### <a name="extended_stdlibscript"></a>`extended_stdlib::script`

Manages a script's existence in the directory of a system, and then executes the script if the script's attributes changed.

#### Examples

##### Manages a script named 'myscript.sh' on the target system and sourced from the containing module, and then executes the script with the default shell interpreter.

```puppet
extended_stdlib::script { 'myscript.sh': }
```

#### Parameters

The following parameters are available in the `extended_stdlib::script` defined type:

* [`script`](#script)
* [`module`](#module)
* [`shell_path`](#shell_path)
* [`stage_dir`](#stage_dir)
* [`source_file`](#source_file)
* [`epp_vars`](#epp_vars)
* [`file_attr`](#file_attr)
* [`exec_attr`](#exec_attr)

##### <a name="script"></a>`script`

Data type: `String`

The filename of the script to be managed and executed. Defaults to the title of the resource.

Default value: `$title`

##### <a name="module"></a>`module`

Data type: `String`

The module containing the script. Defaults to the module where this resource is declared.

Default value: `$caller_module_name`

##### <a name="shell_path"></a>`shell_path`

Data type: `String`

The path to the shell interpreter executable used for executing the managed script file.

Default value: `'/bin/sh'`

##### <a name="stage_dir"></a>`stage_dir`

Data type: `String`

The path to the staging directory for managing the script file on the target system prior to execution.

Default value: `'/tmp'`

##### <a name="source_file"></a>`source_file`

Data type: `String`

The filename of the source script file. Templates are handled automatically. Defaults to the script param.

Default value: `$script`

##### <a name="epp_vars"></a>`epp_vars`

Data type: `Hash`

A hash of variable names and values to be passed as an argument to the epp function (if applicable).

Default value: `{}`

##### <a name="file_attr"></a>`file_attr`

Data type: `Hash`

A hash of additional attribute => value pairs to append to the file resource that manages the script.

Default value: `{}`

##### <a name="exec_attr"></a>`exec_attr`

Data type: `Hash`

A hash of additional attribute => value pairs to append to the exec resource that execures the script.

Default value: `{}`

## Functions

### <a name="extended_stdlibascii_only"></a>`extended_stdlib::ascii_only`

Type: Ruby 4.x API

Returns true for a string which has only ASCII characters, and false otherwise.

#### Examples

##### Returns that a string is ASCII only.

```puppet
ascii_only('abc') => true
```

##### Returns that a string is not ASCII only.

```puppet
ascii_only('abc\u{6666}') => false
```

#### `extended_stdlib::ascii_only(String $a_string, Optional[String] $encoding)`

Returns true for a string which has only ASCII characters, and false otherwise.

Returns: `Boolean` Boolean Whether the string is ASCII only.

##### Examples

###### Returns that a string is ASCII only.

```puppet
ascii_only('abc') => true
```

###### Returns that a string is not ASCII only.

```puppet
ascii_only('abc\u{6666}') => false
```

##### `a_string`

Data type: `String`

The string to determine whether it is ASCII only.

##### `encoding`

Data type: `Optional[String]`

The text encoding for the string (default: UTF-8).

### <a name="extended_stdlibcenter"></a>`extended_stdlib::center`

Type: Ruby 4.x API

Centers a string in width. If the width is greater than the length of the string, then the string is centered by padding and padded with padstring; otherwise the string is returnd.

#### Examples

##### Returns the string since width is smaller than the string.

```puppet
center('hello', 4) => 'hello'
```

##### Returns the centered string padded with empty string.

```puppet
center('hello', 20) => '       hello       '
```

##### Returns the centered string padded with '#-'.

```puppet
center('hello', 20, '#-') => '#-#-#-#hello#-#-#-#'
```

#### `extended_stdlib::center(String $a_string, Integer $width, Optional[String] $pad_string)`

Centers a string in width. If the width is greater than the length of the string, then the string is centered by padding and padded with padstring; otherwise the string is returnd.

Returns: `String` String The resultant apdded string.

##### Examples

###### Returns the string since width is smaller than the string.

```puppet
center('hello', 4) => 'hello'
```

###### Returns the centered string padded with empty string.

```puppet
center('hello', 20) => '       hello       '
```

###### Returns the centered string padded with '#-'.

```puppet
center('hello', 20, '#-') => '#-#-#-#hello#-#-#-#'
```

##### `a_string`

Data type: `String`

The string to center in width with the pad_string.

##### `width`

Data type: `Integer`

The width of the resultant padded string.

##### `pad_string`

Data type: `Optional[String]`

The optional string to use for the surrounding padding.

### <a name="extended_stdlibcompact"></a>`extended_stdlib::compact`

Type: Ruby 4.x API

Returns a hash or array with all of the undef value entries removed

#### Examples

##### Input a hash and return it compacted.

```puppet
compact({'foo' => 0, 'bar' => undef, 'baz' => 2, 'bat' => undef}) => {'foo' => 0, 'baz' => 2}
```

##### Input an array and return it compacted.

```puppet
compact([undef, 0, undef, 1, undef, 2, undef]) => [0, 1, 2]
```

#### `extended_stdlib::compact(Variant[Hash, Array] $hash_or_array)`

Returns a hash or array with all of the undef value entries removed

Returns: `Variant[Hash, Array]` The hash or array with all of the undef value entries removed.

##### Examples

###### Input a hash and return it compacted.

```puppet
compact({'foo' => 0, 'bar' => undef, 'baz' => 2, 'bat' => undef}) => {'foo' => 0, 'baz' => 2}
```

###### Input an array and return it compacted.

```puppet
compact([undef, 0, undef, 1, undef, 2, undef]) => [0, 1, 2]
```

##### `hash_or_array`

Data type: `Variant[Hash, Array]`

The hash or array from which to remove the undef value entries.

### <a name="extended_stdlibdecrypt_string"></a>`extended_stdlib::decrypt_string`

Type: Ruby 4.x API

Returns a decrypted String using the AES-256-CBC OpenSSL cipher algorithm.

#### Examples

##### Decrypt an encrypted string and return it plain text.

```puppet
decrypt_string('/path/to/key.txt', '/path/to/nonce.txt', '/path/to/encrypted.txt') => foobar
```

#### `extended_stdlib::decrypt_string(Pattern[/\A\/([^\n\/\0]+\/*)*\z/] $key, Pattern[/\A\/([^\n\/\0]+\/*)*\z/] $nonce, String $encrypted)`

Returns a decrypted String using the AES-256-CBC OpenSSL cipher algorithm.

Returns: `String` String Returns the encrypted string as decrypted in plain text.

##### Examples

###### Decrypt an encrypted string and return it plain text.

```puppet
decrypt_string('/path/to/key.txt', '/path/to/nonce.txt', '/path/to/encrypted.txt') => foobar
```

##### `key`

Data type: `Pattern[/\A\/([^\n\/\0]+\/*)*\z/]`

The path to the file containing the cipher key.

##### `nonce`

Data type: `Pattern[/\A\/([^\n\/\0]+\/*)*\z/]`

The path to the file containing the cipher nonce.

##### `encrypted`

Data type: `String`

The encrypted string to decrypt.

### <a name="extended_stdlibdrop"></a>`extended_stdlib::drop`

Type: Ruby 4.x API

Returns an Array containing all but the first num element of the array, where num is a non-negative Integer

#### Examples

##### Remove the first 0, 1, or 2 elements from an array, and return that array.

```puppet
drop([0, 1, 2, 3, 4, 5], 0) => [0, 1, 2, 3, 4, 5]
drop([0, 1, 2, 3, 4, 5], 1) => [1, 2, 3, 4, 5]
drop([0, 1, 2, 3, 4, 5], 2) => [2, 3, 4, 5]
```

#### `extended_stdlib::drop(Array $the_array, Integer $num)`

Returns an Array containing all but the first num element of the array, where num is a non-negative Integer

Returns: `Array` Returns the array with the first num elements removed.

##### Examples

###### Remove the first 0, 1, or 2 elements from an array, and return that array.

```puppet
drop([0, 1, 2, 3, 4, 5], 0) => [0, 1, 2, 3, 4, 5]
drop([0, 1, 2, 3, 4, 5], 1) => [1, 2, 3, 4, 5]
drop([0, 1, 2, 3, 4, 5], 2) => [2, 3, 4, 5]
```

##### `the_array`

Data type: `Array`

The array to remove the first num elements from.

##### `num`

Data type: `Integer`

The number of first elements to remove from the array.

### <a name="extended_stdlibend_with"></a>`extended_stdlib::end_with`

Type: Ruby 4.x API

Returns true if the string ends with one of the suffixes given.

#### Examples

##### Check if the string ends with one suffix.

```puppet
end_with('hello', ['ello']) => true
```

##### Check if the string ends with one of the suffixes.

```puppet
end_with('hello', ['heaven', 'ello']) => true
end_with('hello', ['heaven', 'paradise']) => false
```

#### `extended_stdlib::end_with(String $a_string, Array[String] *$suffixes)`

Returns true if the string ends with one of the suffixes given.

Returns: `Boolean` Boolean Whether or not the string ends with one of the given suffixes.

##### Examples

###### Check if the string ends with one suffix.

```puppet
end_with('hello', ['ello']) => true
```

###### Check if the string ends with one of the suffixes.

```puppet
end_with('hello', ['heaven', 'ello']) => true
end_with('hello', ['heaven', 'paradise']) => false
```

##### `a_string`

Data type: `String`

The string to check if it ends with one of the given suffixes.

##### `*suffixes`

Data type: `Array[String]`

The suffixes to check for if the string ends.

### <a name="extended_stdlibimperative"></a>`extended_stdlib::imperative`

Type: Puppet Language

THIS FUNCTION IS CURRENTLY IN BETA. Puppet function to simulate imperative execution for a subset of resources by constructing iterative dependencies of each resource upon the previous resource. This thusly ensures consecutive resource application vis a vis imperative application instead of declarative.

#### Examples

##### Consecutively apply package resources.

```puppet
extended_stdlib::imperative([Package['first'], Package['second'], Package['third']])
```

##### Consecutively apply package resources at latest versions.

```puppet
extended_stdlib::imperative(
  { 'first' => {}, 'second' => {}, 'third' => {} },
  'package',
  { 'ensure' => 'latest' }
)
```

#### `extended_stdlib::imperative(Variant[Hash, Array[Resource]] $resources, Optional[Regexp[/^[a-z]+$/]] $type = '', Optional[Hash] $defaults = {})`

The extended_stdlib::imperative function.

Returns: `Undef`

##### Examples

###### Consecutively apply package resources.

```puppet
extended_stdlib::imperative([Package['first'], Package['second'], Package['third']])
```

###### Consecutively apply package resources at latest versions.

```puppet
extended_stdlib::imperative(
  { 'first' => {}, 'second' => {}, 'third' => {} },
  'package',
  { 'ensure' => 'latest' }
)
```

##### `resources`

Data type: `Variant[Hash, Array[Resource]]`

The hash of resource names and attributes, or array of resources, to construe dependencies for imperative application, and also to declare if input type is hash.

##### `type`

Data type: `Optional[Regexp[/^[a-z]+$/]]`

The resource type to use for declaration if $resources is hash type.

##### `defaults`

Data type: `Optional[Hash]`

The hash of default attributes to use for declaration if $resources is hash type.

### <a name="extended_stdlibintersect"></a>`extended_stdlib::intersect`

Type: Ruby 4.x API

Returns true if the array and other_array have at least one element in common, otherwise returns false. Requires Ruby >= 3.0.

#### Examples

##### Determine if first array and second array intersect.

```puppet
intersect([1, 2, 3], [3, 4, 5]) => true
intersect([1, 2, 3], [5, 6, 7]) => false
```

#### `extended_stdlib::intersect(Array $a_array, Array $other_array)`

Returns true if the array and other_array have at least one element in common, otherwise returns false. Requires Ruby >= 3.0.

Returns: `Boolean` Returns whether the two arrays intersect.

##### Examples

###### Determine if first array and second array intersect.

```puppet
intersect([1, 2, 3], [3, 4, 5]) => true
intersect([1, 2, 3], [5, 6, 7]) => false
```

##### `a_array`

Data type: `Array`

The first array to determine if intersects.

##### `other_array`

Data type: `Array`

The second array to determine if intersects.

### <a name="extended_stdlibminmax"></a>`extended_stdlib::minmax`

Type: Ruby 4.x API

Returns a new two element Array containing the minimum and maximum values from an array of integers

#### Examples

##### Return the minimum and maximum elements of an array.

```puppet
minmax([0, 1, 2]) => [0, 2]
```

#### `extended_stdlib::minmax(Array[Integer, 2] $the_array)`

Returns a new two element Array containing the minimum and maximum values from an array of integers

Returns: `Array[Integer, 2]` Returns two element array of integers with minimum and maximum values.

##### Examples

###### Return the minimum and maximum elements of an array.

```puppet
minmax([0, 1, 2]) => [0, 2]
```

##### `the_array`

Data type: `Array[Integer, 2]`

The array of integers from which to return the minimum and maximum values.

### <a name="extended_stdlibnone"></a>`extended_stdlib::none`

Type: Ruby 4.x API

Returns a boolean of whether no elements of an array, or array or hash with a lambda block meet a given criterion. Note that the combined block input forms do not have code coverage at the moment.

#### `extended_stdlib::none(Array $the_array)`

The extended_stdlib::none function.

Returns: `Boolean` Returns whether no elements satisfy a conditional (i.e. every element is falsey).

##### Examples

###### Return whether no element of any array is truthy.

```puppet
none([undef, false]) => true
none([undef, 0, false]) => false
none([]) => true
```

###### Return whether there is an element with an even numbered index that has a String value (no element satisfies these conditions)

```puppet
['key1', 1, 2, 2].none |$index, $value| { $index % 2 == 0 and $value =~ String } => false
```

###### Return whether there is an element with an even numbered index that does not have a String value (element at index 2 causes 'false' return by satisfying both conditionals)

```puppet
['key1', 1, 2, 2].none |$index, $value| { $index % 2 == 0 and $value !~ String } => false
```

##### `the_array`

Data type: `Array`

The array containing the elements to test the criterion upon. the_hash The hash containing the entries to test the criterion upon. block The optional lambda block to specify the criterion (otherwise directly tests falsiness).

#### `extended_stdlib::none(Hash[Any, Any] $the_hash, Callable[1,1] &$block)`

The extended_stdlib::none function.

Returns: `Boolean`

##### `the_hash`

Data type: `Hash[Any, Any]`



##### `&block`

Data type: `Callable[1,1]`



#### `extended_stdlib::none(Hash[Any, Any] $the_hash, Callable[2,2] &$block)`

The extended_stdlib::none function.

Returns: `Boolean`

##### `the_hash`

Data type: `Hash[Any, Any]`



##### `&block`

Data type: `Callable[2,2]`



#### `extended_stdlib::none(Iterable $enumerable, Callable[1,1] &$block)`

The extended_stdlib::none function.

Returns: `Boolean`

##### `enumerable`

Data type: `Iterable`



##### `&block`

Data type: `Callable[1,1]`



#### `extended_stdlib::none(Iterable $enumerable, Callable[2,2] &$block)`

The extended_stdlib::none function.

Returns: `Boolean`

##### `enumerable`

Data type: `Iterable`



##### `&block`

Data type: `Callable[2,2]`



### <a name="extended_stdlibproduct"></a>`extended_stdlib::product`

Type: Ruby 4.x API

Computes and returns all combinations of elements from all of the Arrays

#### Examples

##### Returns original Array if only one is specified.

```puppet
product([[0, 1, 2]]) => [[0, 1, 2]]
```

##### Return the product of two Arrays.

```puppet
product([[0, 1, 2], [3, 4]]) => [[0, 3], [0, 4], [1, 3], [1, 4], [2, 3], [2, 4]]
```

##### Return the product of three Arrays.

```puppet
product([[0, 1, 2], [3, 4], [5, 6]]) => [[0, 3, 5], [0, 3, 6], [0, 4, 5], [0, 4, 6], [1, 3, 5], [1, 3, 6], [1, 4, 5], [1, 4, 6], [2, 3, 5], [2, 3, 6], [2, 4, 5], [2, 4, 6]]
```

#### `extended_stdlib::product(Array *$arrays)`

Computes and returns all combinations of elements from all of the Arrays

Returns: `Array[Array]` The nested Array containing the product of the Arrays' elements.

##### Examples

###### Returns original Array if only one is specified.

```puppet
product([[0, 1, 2]]) => [[0, 1, 2]]
```

###### Return the product of two Arrays.

```puppet
product([[0, 1, 2], [3, 4]]) => [[0, 3], [0, 4], [1, 3], [1, 4], [2, 3], [2, 4]]
```

###### Return the product of three Arrays.

```puppet
product([[0, 1, 2], [3, 4], [5, 6]]) => [[0, 3, 5], [0, 3, 6], [0, 4, 5], [0, 4, 6], [1, 3, 5], [1, 3, 6], [1, 4, 5], [1, 4, 6], [2, 3, 5], [2, 3, 6], [2, 4, 5], [2, 4, 6]]
```

##### `*arrays`

Data type: `Array`

One or more Arrays of which to compute the products. Issue with required_repeated_param in Puppet requires this to be nested Array.

### <a name="extended_stdlibsample"></a>`extended_stdlib::sample`

Type: Ruby 4.x API

Returns random elements from an Array. Note that the return is non-deterministic.

#### Examples

##### Return a random element from an Array.

```puppet
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]) => 3
```

##### Return multiple random elements from an Array.

```puppet
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 3) => [8, 9, 2]
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 6) => [9, 6, 10, 3, 1, 4]
```

#### `extended_stdlib::sample(Array[1] $the_array, Optional[Integer] $num_elements)`

Returns random elements from an Array. Note that the return is non-deterministic.

Returns: `Variant[Array, Any]` The random element or Array of random elements selected from the Array.

##### Examples

###### Return a random element from an Array.

```puppet
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]) => 3
```

###### Return multiple random elements from an Array.

```puppet
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 3) => [8, 9, 2]
sample([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 6) => [9, 6, 10, 3, 1, 4]
```

##### `the_array`

Data type: `Array[1]`

The Array from which to select random elements.

##### `num_elements`

Data type: `Optional[Integer]`

The number of random elements to select from the Array.

### <a name="extended_stdlibstart_with"></a>`extended_stdlib::start_with`

Type: Ruby 4.x API

Returns true if the string starts with one of the prefixes given. Each prefix should be a String or (Ruby >= 2.5) Regexp.

#### Examples

##### Check if the string begins with one String prefix.

```puppet
start_with('hello', ['hell']) => true
```

##### Check if the string begins with one Regexp prefix.

```puppet
start_with('hello', [/h/]) => true
```

##### Check if the string begins with one of the prefixes.

```puppet
start_with('hello', ['heaven', 'hell']) => true
start_with('hello', ['heaven', 'paradise']) => false
```

#### `extended_stdlib::start_with(String $a_string, Array[Variant[String, Regexp]] *$prefixes)`

Returns true if the string starts with one of the prefixes given. Each prefix should be a String or (Ruby >= 2.5) Regexp.

Returns: `Boolean` Boolean Whether or not the string begins with one of the given prefixes.

##### Examples

###### Check if the string begins with one String prefix.

```puppet
start_with('hello', ['hell']) => true
```

###### Check if the string begins with one Regexp prefix.

```puppet
start_with('hello', [/h/]) => true
```

###### Check if the string begins with one of the prefixes.

```puppet
start_with('hello', ['heaven', 'hell']) => true
start_with('hello', ['heaven', 'paradise']) => false
```

##### `a_string`

Data type: `String`

The string to check if it begins with one of the given prefixes.

##### `*prefixes`

Data type: `Array[Variant[String, Regexp]]`

The prefixes to check for if the string begins.

### <a name="extended_stdlibsubset"></a>`extended_stdlib::subset`

Type: Ruby 4.x API

Returns a boolean of whether the first hash is a subset of the second hash

#### Examples

##### Determine if hash is subset of other_hash.

```puppet
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1, 'baz' => 2}) => true
subset({'foo' => 0, 'bar' => 1, 'baz' => 2}, {'foo' => 0, 'bar' => 1}) => false
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1}) => true
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1}, true) => false
```

#### `extended_stdlib::subset(Hash $a_hash, Hash $other_hash, Optional[Boolean] $proper)`

Returns a boolean of whether the first hash is a subset of the second hash

Returns: `Boolean` Returns whether hash is a subset of other_hash.

##### Examples

###### Determine if hash is subset of other_hash.

```puppet
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1, 'baz' => 2}) => true
subset({'foo' => 0, 'bar' => 1, 'baz' => 2}, {'foo' => 0, 'bar' => 1}) => false
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1}) => true
subset({'foo' => 0, 'bar' => 1}, {'foo' => 0, 'bar' => 1}, true) => false
```

##### `a_hash`

Data type: `Hash`

The hash to determine if this hash is a subset of other_hash.

##### `other_hash`

Data type: `Hash`

The hash to determine if hash is a subset of this hash.

##### `proper`

Data type: `Optional[Boolean]`

Specifies if the subset determination should be for a proper subset or regular subset.

### <a name="extended_stdlibsum"></a>`extended_stdlib::sum`

Type: Ruby 4.x API

Returns the sum of an optional summand with all elements of an Array.

#### Examples

##### Returns sum of Array of Integers.

```puppet
sum([0, 1, 2, 3]) => 6
```

##### Returns sum of Integer and Array of Integers.

```puppet
sum([0, 1, 2, 3], 100) => 106
```

##### Returns sum of String and Array of Strings.

```puppet
sum(['abc', 'def', 'ghi'], 'jkl') => 'jklabcdefghi'
```

#### `extended_stdlib::sum(Array[Variant[Numeric, String]] $the_array, Optional[Variant[Numeric, String]] $summand)`

Returns the sum of an optional summand with all elements of an Array.

Returns: `Variant[Numeric, String]`

##### Examples

###### Returns sum of Array of Integers.

```puppet
sum([0, 1, 2, 3]) => 6
```

###### Returns sum of Integer and Array of Integers.

```puppet
sum([0, 1, 2, 3], 100) => 106
```

###### Returns sum of String and Array of Strings.

```puppet
sum(['abc', 'def', 'ghi'], 'jkl') => 'jklabcdefghi'
```

##### `the_array`

Data type: `Array[Variant[Numeric, String]]`

The array to sum with an optional summand.

##### `summand`

Data type: `Optional[Variant[Numeric, String]]`

The optional summand to sum with the Array elements.

### <a name="extended_stdlibtranspose"></a>`extended_stdlib::transpose`

Type: Ruby 4.x API

Transposes the rows and columns in an Array of Arrays; the nested Arrays must all be the same size.

#### Examples

##### Returns 1x2 transposition of 2x1 nested Array.

```puppet
transpose([['a0', 'a1']]) => [['a0'], ['a1']]
```

##### Returns 3x2 transposition of 2x3 nested Array.

```puppet
transpose([['a0', 'a1'], ['b0', 'b1'], ['c0', 'c1']]) => [['a0', 'b0', 'c0'], ['a1', 'b1', 'c1']]
```

#### `extended_stdlib::transpose(Array[NotUndef, 1] *$arrays)`

Transposes the rows and columns in an Array of Arrays; the nested Arrays must all be the same size.

Returns: `Array[Array]` The nested Array containing the transposition of the Arrays' elements.

##### Examples

###### Returns 1x2 transposition of 2x1 nested Array.

```puppet
transpose([['a0', 'a1']]) => [['a0'], ['a1']]
```

###### Returns 3x2 transposition of 2x3 nested Array.

```puppet
transpose([['a0', 'a1'], ['b0', 'b1'], ['c0', 'c1']]) => [['a0', 'b0', 'c0'], ['a1', 'b1', 'c1']]
```

##### `*arrays`

Data type: `Array[NotUndef, 1]`

One or more equal-size Arrays of which to transpose. Issue with required_repeated_param in Puppet requires this to be nested Array.

## Tasks

### <a name="csr_attributes"></a>`csr_attributes`

Modifies CSR attributes on a client server.

**Supports noop?** false

#### Parameters

##### `extension_requests`

Data type: `Hash`

The desired extension requests (permanent data to be embedded in a signed certificate). Note that input values will overwrite current values with the same paired key.

##### `custom_attributes`

Data type: `Hash`

The desired custom attributes (transient data used for pre-validating requests). Note that input values will overwrite current values with the same paired key.

##### `purge_extension_requests`

Data type: `Boolean`

Whether or not to purge the existing extension requests in the CSR attributes when updating.

##### `purge_custom_attributes`

Data type: `Boolean`

Whether or not to purge the existing custom attributes in the CSR attributes instead when updating.

## Plans

### <a name="extended_stdlibcsr_regenerate"></a>`extended_stdlib::csr_regenerate`

This plan modifies the CSR attributes on servers, and then regenerates the client certificate on the servers to enable PuppetDB to update the values from the CSR attributes.

#### Parameters

The following parameters are available in the `extended_stdlib::csr_regenerate` plan:

* [`servers`](#servers)
* [`puppet_orchestrator`](#puppet_orchestrator)
* [`extension_requests`](#extension_requests)
* [`custom_attributes`](#custom_attributes)
* [`purge_extension_requests`](#purge_extension_requests)
* [`purge_custom_attributes`](#purge_custom_attributes)

##### <a name="servers"></a>`servers`

Data type: `TargetSpec`

The servers to target for CSR attribute modification.

##### <a name="puppet_orchestrator"></a>`puppet_orchestrator`

Data type: `TargetSpec`

The server hosting the Puppet Orchestrator software for executing plans and tasks.

##### <a name="extension_requests"></a>`extension_requests`

Data type: `Hash`

The desired extensions requests in the CSR attributes.

Default value: `{}`

##### <a name="custom_attributes"></a>`custom_attributes`

Data type: `Hash`

The desired custom attributes in the CSR attributes.

Default value: `{}`

##### <a name="purge_extension_requests"></a>`purge_extension_requests`

Data type: `Boolean`

Whether or not to purge the existing extension requests in the CSR attributes when updating.

Default value: ``false``

##### <a name="purge_custom_attributes"></a>`purge_custom_attributes`

Data type: `Boolean`

Whether or not to purge the existing custom attributes in the CSR attributes instead when updating.

Default value: ``false``

